#!/usr/bin/env/python
# Tells Linux systems how to run this file.

"""
Morningside AI Alpha
Purpose: Demonstrate minimal interaction with worldsim "Morningside".

Overall structure:
This program runs a main loop that checks whether an input file exists at
the designated path: "from_"+self.name.lower()+".txt" in a certain directory.
If there's such a file, it reads that and processes it as a dictionary object.
This minimal program does nothing with the info other than that processing
and printing the result.
It then creates an output file containing a random command.

How to use it:
Run the Morningside server program, then run this. This program will make
the default character (Azure) randomly wander for 10 turns.
This is assuming that you've left a "login_azure.txt" in the player_io directory of Morningside,
so that a character called Azure will be created.
Be sure that the path below, in the constants section, points to the player_io
directory.
"""

__author__ = "Kris Schnee"
__version__ = "2009.9.26"
__license__ = "Public Domain"

##### Imported Modules #####

## Standard
import os
import time
import random

## Third-party


## Mine


##### Constants #####

"""
The path needs to reference the Morningside program's player_io folder.
Make sure that a file called path.txt is in this AI program's directory, and
that it consists of the appropriate path, eg. "C:\Code\Morningside\player_io".
"""
f = open("path.txt","r")
path = f.read()
f.close()
WORLDSIM_IO_PATH = path

"""When reading input files, input values with certain keys should be treated
as integers."""
INPUT_INT_KEYS = []

##### Classes #####

class DemoAI:
    def __init__(self,**options):
        self.name = options.get("name","AI")
        self.input_search_cycles = 0 ## Completed cycles through main loop
        self.ai_cycles = 0

        """Filenames used for IO.
        The AI will look for text files in a certain path as input,
        and send its output as a text file in that same directory."""
        
        self.input_filename = "to_"+self.name.lower()+".txt"
        self.output_filename = "from_"+self.name.lower()+".txt"
        self.input_path = os.path.join(WORLDSIM_IO_PATH,self.input_filename)
        self.output_path = os.path.join(WORLDSIM_IO_PATH,self.output_filename)
        self.action = None ## What shall I do this round?

    def CheckForInput(self):
        self.input_search_cycles += 1
        print "Searching for input file (search #"+str(self.input_search_cycles)+")"
        if os.path.exists(self.input_path):
            f = open(self.input_path,"r")
            text = f.read()
            f.close()
            os.remove(self.input_path)
            return text
        return None

    def UnpackInput(self,text=""):
        """Read input from a Morningside worldsim file inputted to me.

        File format:
        Each line is one of these, determined by what it starts with:
        # Comment
        * News (something that happened)
        @ Entity
        $ System (info like "this rule is in play")
        In each case, character 0 is that marker and 1 is a blank space.
        """
        
        lines = [l for l in text.split("\n") if l and l[0] != "#"]
        result = {"news":[],"entities":[],"system":[]}
        for line in lines:
            ## Split into key and value, as with the Python dict format.
            s = line[2:].split(":")
            if len(s) > 2:
                ## This line uses an extra colon somewhere. Messy to deal with.
                f = line.find(":")
                k, v = line[2:f], line[f+1:]
            else:
                k, v = s
            v = v.lstrip(" ")

            ## Store the info based on what type this line is.

            if line[0] == "@": ## eg. "@ 4,2: 007 coin"
                k = k.split(",")
                k = int(k[0]),int(k[1]) ## eg. (4,2)
                v = v.split(" ")
                e = {"ID":v[0],
                     "name":v[1],
                     "alive":len(v)>2 and v[2].lower()=="alive"
                     }
                result["entities"].append(e)

            elif line[0] == "*": ## eg. "*5: get ok"
                result["news"].append((k,v))

            elif line[0] == "$": ## eg. "1: you"
                result["system"].append((k,v))

        return result

    def PickAction(self):
        """INSERT BRAIN HERE"""
        self.ai_cycles += 1
        n = random.randint(0,4)
        if n == 4:
            self.action = ("say","Hello World!")
        else:
            self.action = ("go",["e","n","w","s"][n])

    def WriteOutput(self):
        if not self.action:
            return
        text = " ".join(self.action)
        f = open(self.output_path,"w")        
        f.write(text)
        f.close()

    def MainLoop(self):
        max_cycles = 50
        max_ai_cycles = 10
        while True:
            text = self.CheckForInput()
            if text:
##                print "Got an input file. Raw input follows:"
##                print text
                result = self.UnpackInput(text)

                ## Do something with the input. For this demo, just print it!
                print "Unpacked input:"
                for key in result:
                    print key+": "+str(result[key])

                self.PickAction()
                self.WriteOutput()

            if self.input_search_cycles == max_cycles:
                print "Max search cycles reached. Ending demo."
                break
            elif self.ai_cycles == max_ai_cycles:
                print "Max AI cycles reached. Ending demo."
                break

            time.sleep(1)
            
        print "Done."


##### Other Functions #####



##### Autorun #####

if __name__ == "__main__":
    ## Runs if this module is the main module (not imported).
    ai = DemoAI(name="Azure")
    ai.MainLoop()
